import requests
import feedparser
from typing import List
from readability import Document
import multiprocessing

from .story import Story
from .storyprovider import StoryProvider


def parallelizable_request(entry):
    req = requests.get(entry["link"])
    if not req.ok:
        print(f"Honk! Couldn't get content for {entry['link']}")
        return None

    doc = Document(req.content)
    source = entry["link"].split(".")[1]
    story = Story(doc.title(), body_html=doc.summary(), byline=source)

    return story


class RSSFeedStoryProvider(StoryProvider):
    def __init__(
        self,
        rss_path: str,
        limit: int = 5,
        parallel: bool = True,
    ) -> None:
        self.limit = limit
        self.feed_url = rss_path
        self._parallel = parallel

    def get_stories(self, limit: int = 5) -> List[Story]:
        feed = feedparser.parse(self.feed_url)
        limit = min(limit, self.limit, len(feed.entries))
        if limit == 0:
            print(f"Sad honk :/ No entries found for feed {self.feed_url}...")

        if self._parallel:
            with multiprocessing.Pool(multiprocessing.cpu_count()) as pool:
                stories = pool.map(
                    parallelizable_request,
                    feed.entries[:limit],
                )
        else:
            stories = [parallelizable_request(e) for e in feed.entries[:limit]]

        return list(filter(None, stories))



import abc
from typing import List
from .story import Story


class StoryProvider(abc.ABC):
    """
    An abstract class for a class that provides stories to be rendered.
    """

    def get_stories(self, limit: int = 5) -> List["Story"]:
        """
        Get a list of stories from this Provider.
        """
        ...


class LoremStoryProvider(StoryProvider):
    def __init__(self, limit: int = 5):
        self.text = [
            "Lorem ipsum! dolor sit amet, consectetur adipiscing elit. Duis eget velit sem. In elementum eget lorem non luctus. Vivamus tempus justo in pulvinar ultrices. Aliquam ac maximus leo. Quisque ipsum sapien, vestibulum viverra tempus ac, vestibulum quis justo. Nullam ut purus varius, bibendum metus ac, viverra enim. Phasellus sodales ullamcorper sapien pretium tristique. Duis dapibus felis quis tincidunt ultrices. Etiam purus sapien, tincidunt ac turpis vel, eleifend placerat enim. In sed mauris justo. Suspendisse ac tincidunt nunc. Nullam luctus porta pretium. Donec porttitor, nulla ut finibus pretium, augue turpis posuere ante, ac congue nunc nulla eu nisl. Phasellus imperdiet vel augue id gravida.",
            "Morbi mattis egestas quam, in tempus elit efficitur sagittis. Sed in maximus lorem. Aliquam erat volutpat. Phasellus mattis varius velit, vitae varius justo. Sed imperdiet eget dolor non consequat. Cras non felis neque. Nam eget arcu sapien. Morbi ultrices tristique cursus. Sed tempor ex lorem, vel ultrices sem placerat non. Nullam tortor arcu, imperdiet id lobortis a, commodo nec mi. Duis rhoncus in est sit amet tristique. Mauris condimentum nisl a erat tristique, id dictum risus euismod. Phasellus at sapien ante. Morbi facilisis tortor id leo porta, condimentum mollis dolor suscipit.",
            "Phasellus ut nibh vitae turpis congue venenatis. Morbi mollis justo dolor, ac finibus erat suscipit vitae. Donec libero erat, luctus quis sapien vel, sagittis dapibus est. Ut non quam et nisl hendrerit sodales. Pellentesque habitant morbi tristique senectus et netus et malesuada fames ac turpis egestas. Integer sodales ut augue a lacinia. Phasellus mattis sapien eget nibh auctor porttitor. Sed feugiat consectetur risus, a tempus ipsum scelerisque eu. Maecenas suscipit erat quis neque vulputate, ornare vehicula tellus lobortis. Duis tempor elit scelerisque ex tincidunt imperdiet. Curabitur dictum condimentum turpis, vitae ultrices ante sodales a. Praesent eu erat nec odio placerat placerat. Sed et dolor augue.",
            "Curabitur consectetur, nisi eget consequat ultrices, erat ante tincidunt ipsum, eget varius mauris turpis ac enim. Vivamus rutrum condimentum metus ut egestas. Nulla consectetur tincidunt laoreet. Vivamus tortor sem, imperdiet sodales facilisis quis, elementum nec erat. Curabitur imperdiet, nulla vel mattis gravida, risus eros sollicitudin magna, nec feugiat mauris mauris eu lorem. In hac habitasse platea dictumst. Sed tincidunt facilisis sem, non commodo metus volutpat nec. Fusce nulla mauris, vulputate sit amet magna id, blandit ornare leo. Nam vel faucibus ipsum, ac congue dolor.",
            "Vivamus pretium purus vel libero finibus blandit. Donec vitae nisl sollicitudin, consectetur nunc ac, volutpat libero. Maecenas ac leo ut velit viverra aliquet non id turpis. Morbi ut euismod erat. Vestibulum congue sed erat nec dapibus. Donec semper consectetur vestibulum. Praesent egestas dolor a ante sodales maximus. Suspendisse a odio vitae odio sagittis sollicitudin in quis massa. Praesent at convallis nulla. Mauris a nisl tincidunt, iaculis lacus eget, lobortis sapien. Nullam condimentum neque quis nisi consequat, eget accumsan tellus fermentum. Quisque dictum, nunc et pretium accumsan, lacus eros pharetra odio, ac euismod orci lorem sed turpis. ",
        ]
        self.limit = limit

    def get_stories(self, limit: int = 5):
        return [
            Story(headline="Lorem Ipsum Dolor Sit Amet", body_text=self.text)
            for _ in range(min(self.limit, limit))
        ]



from .util import (
    htmlize,
    clean_html,
    clean_text,
    construct_story_providers_from_config_dict,
)


def test_htmlize():
    assert htmlize(["foo", "bar"]) == "<p>foo</p><p>bar</p>"


def test_clean_html():
    assert clean_html("fooâ€TMbar") == "foo'bar"


def test_clean_text():
    assert clean_text("fooâ€TMbar") == "foo'bar"


def test_construct_story_providers_from_config_dict():
    assert construct_story_providers_from_config_dict({}) == []
    stories = construct_story_providers_from_config_dict(
        {"stories": [{"provider": "twitter", "config": {"usernames": "j6m8"}}]}
    )
    assert len(stories) == 1

    stories = construct_story_providers_from_config_dict(
        {
            "stories": [
                {"provider": "twitter", "config": {"usernames": ["j6m8"]}},
                {"provider": "reddit", "config": {"subreddit": "worldnews"}},
            ]
        }
    )
    assert len(stories) == 2



import requests
from typing import List

from .util import PlacementPreference
from .storyprovider import StoryProvider
from .story import Story


class WeatherStoryProvider(StoryProvider):
    def __init__(self, woe: str = "2358820", F: bool = True):
        self.woe = woe
        self.F = F

    def CtoF(self, temp: float) -> float:
        return (temp * 9 / 5) + 32

    def get_stories(self, limit: int = 1) -> List[Story]:
        weatherReq = requests.get(
            f"https://www.metaweather.com/api/location/{self.woe}/"
        ).json()
        weather = weatherReq["consolidated_weather"][0]
        weatherTitle = weatherReq["title"]
        if self.F:
            headline = f"{int(self.CtoF(weather['the_temp']))}ºF with {weather['weather_state_name']} in {weatherTitle}"
            body_html = f"""
            <img
                src="https://www.metaweather.com/static/img/weather/png/64/{weather['weather_state_abbr']}.png"
                width="42" />
            {int(self.CtoF(weather['min_temp']))} – {int(self.CtoF(weather['max_temp']))}ºF, Winds {weather['wind_direction_compass']}
            """
        else:
            headline = f"{weather['the_temp']:.1f}ºC with {weather['weather_state_name']} in {weatherTitle}"
            body_html = f"""
            <img
                src="https://www.metaweather.com/static/img/weather/png/64/{weather['weather_state_abbr']}.png"
                width="42" />
            {weather['min_temp']:.1f} – {weather['max_temp']:.1f}ºC, Winds {weather['wind_direction_compass']}
            """
        return [
            Story(
                headline=headline,
                body_html=body_html,
                placement_preference=PlacementPreference.EAR,
            )
        ]



import feedparser
import bs4
from typing import List

from .util import PlacementPreference
from .storyprovider import StoryProvider
from .story import Story


class WikipediaCurrentEventsStoryProvider(StoryProvider):
    """
    A story provider that reads from today's current events on Wikipedia.
    """

    def __init__(self):
        pass

    def get_stories(self, limit: int = 10) -> List[Story]:
        """
        Get a list of current stories from Wikipedia.
        """
        feed = feedparser.parse("https://www.to-rss.xyz/wikipedia/current_events/")
        # title = feed.entries[0].title
        title = "Today's Current Events"
        content = bs4.BeautifulSoup(feed.entries[0].summary, "lxml")
        for a in content.find_all("a"):
            while a.find("li"):
                a.find("li").replace_with_children()
            while a.find("ul"):
                a.find("ul").replace_with_children()
            a.replace_with_children()

        while content.find("dl"):
            content.find("dl").name = "h3"
        return [
            Story(
                headline=title,
                body_html=str(content),
                byline="Wikipedia Current Events",
                placement_preference=PlacementPreference.BANNER,
            )
        ]



from rmapy.api import Client
from rmapy.exceptions import AuthError

CODE_URL = "https://my.remarkable.com/connect/remarkable"


def auth_client():
    client = Client()

    try:
        client.renew_token()
    except AuthError:
        print(
            "Looks like this is the first time you've uploaded. You need to "
            f"register the device. Input a code from {CODE_URL}"
        )
        code = input()
        print("registering")
        client.register_device(code)
        if not client.renew_token():
            print("Honk! Registration renewal failed.")
            return False
        else:
            print("registration successful")

    return client



class Style:
    def get_stylesheets(self) -> list:
        ...

    def get_css(self, font_size: int = 14):
        ...


class AutumnStyle(Style):
    def get_stylesheets(self) -> list:
        return [
            "https://fonts.googleapis.com/css?family=Oswald",
            "https://fonts.googleapis.com/css?family=Playfair+Display",
        ]

    def get_css(self, font_size: int = 14):
        return (
            """
        @page {
            margin-top: 0.5in;
            margin-right: 0.2in;
            margin-left: 0.65in;
        }

        body {
            font-family: "Playfair Display";
        }

        .header {
            padding: 1em;
            height: 10em;
        }

        .header div {
            float: left;
            display: block;
        }

        .header .ear {
            float: right;
        }

        ul, li, ol {
            margin-left: 0; padding-left: 0.15em;
        }

        .stories {
            font-size: """
            + str(font_size)
            + """pt;
        }

        .ear article {
            border: 1px groove black;
            padding: 1em;
            margin: 1em;
            font-size: 11pt;
        }
        .ear article h1 {
            font-family: "Playfair Display";
            font-size: 10pt;
            font-weight: normal;
        }

        article {
            text-align: justify;
            line-height: 1.25em;
        }

        .longform {
            page-break-after: always;
        }

        article>h1 {
            font-family: "Oswald";
            font-weight: 400;
            font-size: 23pt;
            text-indent: 0;
            margin-bottom: 0.25em;
            line-height: 1.2em;
            text-align: left;
        }
        article>h1.priority-low {
            font-family: "Oswald";
            font-size: 18pt;
            font-weight: 400;
            text-indent: 0;
            border-bottom: 1px solid #dedede;
            margin-bottom: 0.15em;
        }

        article>h4.byline {
            font-family: "Oswald";
            font-size: """
            + str(font_size)
            + """pt;
            font-weight: 400;
            text-indent: 0;
            border-bottom: 1px solid #dedede;
        }

        article>h3 {
            font-family: "Oswald";
            font-weight: 400;
            font-size: 18pt;
            text-indent: 0;
        }

        section>h1,
        section>h2,
        section>h3,
        section>h4,
        section>h5 {
            border-left: 5px solid #dedede;
            padding-left: 1em;
        }

        figure {
            border: 1px solid black;
            text-indent: 0;
            width: auto;
        }

        .stories article.story img {
            width: 100%;
        }

        figure>span {
            font-size: 0;
        }

        .row {
            column-count: 2;
        }

        """
        )



from goosepaper.multiparser import MultiParser
from rmapy.document import ZipDocument
from rmapy.api import Folder

import os
from pathlib import Path

from .auth import auth_client


def sanitycheck(folder: str, client):

    # First lets do a sanity check. Since RM cloud uses an object ID as a unique key
    # it is entirely possible to have duplicate VissibleName attributes for both folders and
    # documents, and you can have folders and documents that share the same name.

    # This is insanity and we have no programmatic way to resolve it so we're going to cheat.
    # Check for duplicate folder names as the root level and if found we simply force the
    # user to resolve it and check for duplicate document names.

    rootfolders = [
        f
        for f in client.get_meta_items()
        if (f.Type == "CollectionType" and f.Parent == "")
    ]

    uniquefolders = set()
    [
        uniquefolders.add(folder.VissibleName.lower()) or folder
        for folder in rootfolders
        if folder.VissibleName.lower() not in uniquefolders
    ]
    foldercountdif = abs(len(uniquefolders) - len(rootfolders))

    folderduperr = ""

    if foldercountdif == 1:
        folderduperr = "I found a duplicate folder name in the root of your RM.\n"
    elif foldercountdif > 1:
        folderduperr = (
            "You have multiple duplicate folder names in the root of your RM.\n"
        )
    else:
        pass

    if foldercountdif:
        print("{0}\n\nYou must fix this first.".format(folderduperr))
        return False
    else:
        return True


def validateFolder(folder: str):
    if folder:
        if folder == "":
            print("Honk! Folder cannot be an empty string")
            return False
        elif "/" in folder:
            print("Honk! Please do not include '/' in the folder name")
            print("      Nested folders are not supported for now")
            return False

    return True


def getallitems(client):

    # So somehow I corrupted or broke my cloud during testing and any object
    # which exists is getting returned twice in the object list from
    # get_meta_items. Deleting items and re-adding them hasn't fixed it even
    # using the official RM client. So this avoids that problem. I haven't found
    # a real fix yet for my cloud though.

    allitems = [item for item in client.get_meta_items() if (item.Parent != "trash")]

    items = []
    for tempitem in allitems:
        if not any(item.ID == tempitem.ID for item in items):
            items.append(tempitem)

    return items


def upload(filepath, multiparser=None):

    if not multiparser:
        multiparser = MultiParser()

    filepath = Path(filepath)
    replace = (
        False
        if multiparser.argumentOrConfig("noreplace")
        else multiparser.argumentOrConfig("replace")
    )
    folder = multiparser.argumentOrConfig("folder")
    cleanup = multiparser.argumentOrConfig("cleanup")
    strictlysane = multiparser.argumentOrConfig("strictlysane")
    nocase = multiparser.argumentOrConfig("nocase")

    if strictlysane:
        nocase = True

    if multiparser.argumentOrConfig("showconfig"):
        print("\nParameters passed to upload\n----------------\n")
        print(
            "Replace:\t{0}\nFolder:\t\t{1}\nCleanup:\t{2}"
            "\nStrictlysane:\t{3}\nNocase:\t\t{4}\nFilepath:\t{5}\n".format(
                replace, folder, cleanup, strictlysane, nocase, filepath
            )
        )

    client = auth_client()

    if not client:
        print("Honk Honk! Couldn't auth! Is your rmapy configured?")
        return False

    if not validateFolder(folder):
        return False

    # Added error handling to deal with possible race condition where the file
    # is mangled or not written out before the upload actually occurs such as
    # an AV false positive. 'pdf' is a simple throwaway file handle to make
    # sure that we retain control of the file while it's being imported.

    fpr = filepath.resolve()

    try:
        doc = ZipDocument(doc=str(fpr))
    except IOError as err:
        raise IOError(f"Error locating or opening {filepath} during upload.") from err

    paperCandidates = []
    paperFolder = None

    for item in getallitems(client):

        # is it the folder we are looking for?
        if (
            folder
            and item.Type == "CollectionType"  # is a folder
            and item.VissibleName.lower()
            == folder.lower()  # has the name we're looking for
            and (item.Parent is None or item.Parent == "")
        ):  # is not in another folder
            paperFolder = item

        # is it possibly the file we are looking for?
        elif item.Type == "DocumentType" and (
            item.VissibleName.lower() == str(doc.metadata["VissibleName"]).lower()
        ):
            paperCandidates.append(item)

    # TODO: if the folder was found, check if a paper candidate is in it
    # for paper in paperCandidates:
    #     parent = client.get_doc(paper.Parent)

    paper = None
    if len(paperCandidates) > 0:
        if folder:
            filtered = [
                item for item in paperCandidates if item.Parent == paperFolder.ID
            ]
        else:
            filtered = list(
                filter(
                    lambda item: item.Parent != "trash"
                    and client.get_doc(item.Parent) is None,
                    paperCandidates,
                )
            )

        if len(filtered) > 1 and replace:
            print(
                "multiple candidate papers with the same name "
                f"{filtered[0].VissibleName}, don't know which to delete"
            )
            return False
        if len(filtered) == 1:  # found the outdated paper
            paper = filtered[0]

    if paper is not None:
        if replace:
            result = client.delete(paper)
        else:
            print("Honk! The paper already exists!")
            return False

    if folder and not paperFolder:
        paperFolder = Folder(folder)
        if not client.create_folder(paperFolder):
            print("Honk! Failed to create the folder!")
            return False

    # workarround rmapy bug: client.upload(doc) would set a non-existing parent
    # ID to the document
    if not paperFolder:
        paperFolder = Folder()
        paperFolder.ID = ""
    if isinstance(paperFolder, Folder):
        result = client.upload(doc, paperFolder)
        if result:
            print("Honk! Upload successful!")
            if cleanup:
                try:
                    os.remove(fpr)
                except Exception as err:
                    raise IOError(f"Failed to remove file after upload: {fpr}") from err
        else:
            print("Honk! Error with upload!")
        return result
    else:
        print("Honk! Could not upload: Document already exists.")
    return False



import enum
import re
import json
from typing import List, Union


def htmlize(text: Union[str, List[str]]) -> str:
    """
    Generate HTML text from a text string, correctly formatting paragraphs etc.
    """
    # TODO:
    #   * Escaping
    #   * Paragraph delims
    #   * Remove illegal elements
    if isinstance(text, list):
        return "".join([f"<p>{line}</p>" for line in text])
    return f"<p>{text}</p>"


def clean_html(html: str) -> str:
    html = html.replace("â€TM", "'")
    html = re.sub(r"http[s]?:\/\/[^\s\"']+", "", html)
    return html


def clean_text(text: str) -> str:
    text = text.replace("â€TM", "'")
    text = re.sub(r"http[s]?:\/\/[^\s\"']+", "", text)
    return text


class PlacementPreference(enum.Enum):
    NONE = 0
    FULLPAGE = 1
    SIDEBAR = 2
    EAR = 3
    FOLIO = 4
    BANNER = 5


class StoryPriority(enum.Enum):
    DEFAULT = 0
    LOW = 1
    HEADLINE = 5
    BANNER = 9


def load_config_file(filepath: str) -> dict:
    try:
        with open(filepath, "r") as fh:
            config_dict = json.load(fh)
    except ValueError as err:
        raise ValueError(
            "Honk Honk! Syntax Error in config file {0}".format(filepath)
        ) from err
    return config_dict


def construct_story_providers_from_config_dict(config: dict):

    from goosepaper.rss import RSSFeedStoryProvider
    from goosepaper.twitter import MultiTwitterStoryProvider
    from goosepaper.reddit import RedditHeadlineStoryProvider
    from goosepaper.storyprovider import LoremStoryProvider
    from goosepaper.weather import WeatherStoryProvider
    from goosepaper.wikipedia import WikipediaCurrentEventsStoryProvider

    StoryProviderConfigNames = {
        "lorem": LoremStoryProvider,
        "twitter": MultiTwitterStoryProvider,
        "reddit": RedditHeadlineStoryProvider,
        "weather": WeatherStoryProvider,
        "wikipedia_current_events": WikipediaCurrentEventsStoryProvider,
        "rss": RSSFeedStoryProvider,
    }

    if "stories" not in config:
        return []

    stories = []

    for provider_config in config["stories"]:
        provider_name = provider_config["provider"]
        if provider_name not in StoryProviderConfigNames:
            raise ValueError(f"Provider {provider_name} does not exist.")
        if provider_config["config"].get("skip"):
            continue
        else:
            stories.append(
                StoryProviderConfigNames[provider_name](**provider_config["config"])
            )
    return stories



import enum
import twint
from typing import List

import pandas as pd

from .util import clean_text, PlacementPreference
from .storyprovider import StoryProvider
from .story import Story


class TwitterStoryProviderPriorityMode(enum.Enum):
    DEFAULT = 0
    RECENT = 1
    TOP = 2
    RATIO = 3


class TwitterStoryProvider(StoryProvider):
    def __init__(
        self,
        username: str,
        limit: int = 5,
        priority_mode: TwitterStoryProviderPriorityMode = TwitterStoryProviderPriorityMode.DEFAULT,
    ) -> None:
        """
        Create a new TwitterStoryProvider that reads from a username.

        Prioritizes stories based upon the TwitterStoryProviderPriorityMode.
        """
        self.username = username
        self.limit = limit
        self.priority_mode = priority_mode

    def get_stories(self, limit: int = 10) -> List[Story]:
        """
        Get a list of stories.

        Here, the headline is the @username, and the body text is the tweet.
        """

        c = twint.Config()
        c.Pandas = True
        c.Search = f"from:{self.username}"
        c.Limit = min(self.limit, limit)
        c.Hide_output = True

        twint.run.Search(c)
        df = twint.storage.panda.Tweets_df  # type: ignore
        return [
            Story(
                headline=None,
                body_text=clean_text(row.tweet),
                byline=f"@{self.username} on Twitter at {pd.to_datetime(row.date).strftime('%I:%M %p')}",
                date=pd.to_datetime(row.date),
                placement_preference=PlacementPreference.SIDEBAR,
            )
            for i, row in list(df.iterrows())[: min(self.limit, limit)]
        ]


class MultiTwitterStoryProvider(StoryProvider):
    def __init__(
        self,
        usernames: List[str],
        limit_per: int = 5,
        priority_mode: TwitterStoryProviderPriorityMode = TwitterStoryProviderPriorityMode.DEFAULT,
    ) -> None:
        """
        Create a new story provider that reads tweets from several users.

        Arguments:
            usernames (List[str]): A list of twitter usernames
            limit_per (int: 5): A maximum number of tweets to fetch from each
                user in `usernames`
            priority_mode (TwitterStoryProviderPriorityMode): Which priority
                technique to use. Not currently implemented.

        """
        self.usernames = usernames
        self.limit_per = limit_per
        self.priority_mode = priority_mode

    def get_stories(self, limit: int = 42) -> List[Story]:
        """
        Get a list of tweets where each tweet is a story.

        Arguments:
            limit (int: 15): The maximum number of tweets to fetch

        Returns:
            List[Story]: A list of tweets

        """
        stories = []
        for username in self.usernames:
            stories.extend(
                TwitterStoryProvider(
                    username, self.limit_per, self.priority_mode
                ).get_stories(limit=self.limit_per)
            )
        stories = sorted(stories, key=lambda story: story.date)
        return stories[:limit]



import datetime
from typing import List, Optional, Union

from .util import PlacementPreference, htmlize, StoryPriority


class Story:
    def __init__(
        self,
        headline: Optional[str],
        body_html: str = None,
        body_text: Union[str, List[str]] = None,
        byline: str = None,
        date: datetime.datetime = None,
        priority: StoryPriority = StoryPriority.DEFAULT,
        placement_preference: PlacementPreference = PlacementPreference.NONE,
    ) -> None:
        """
        Create a new Story with headline and body text.
        """
        self.headline = headline
        self.priority = priority
        self.byline = byline
        self.date = date
        if body_html is not None:
            self.body_html = body_html
        elif body_text is not None:
            self.body_html = htmlize(body_text)
        else:
            raise ValueError(
                "You must provide at least one of body_html or body_text "
                "to the Story constructor"
            )
        self.placement_preference = placement_preference

    def to_html(self) -> str:
        byline_h4 = f"<h4 class='byline'>{self.byline}</h4>" if self.byline else ""
        priority_class = {
            StoryPriority.DEFAULT: "",
            StoryPriority.LOW: "priority-low",
            StoryPriority.BANNER: "priority-banner",
        }[self.priority]
        headline = (
            f"<h1 class='{priority_class}'>{self.headline}</h1>"
            if self.headline
            else ""
        )
        return f"""
        <article class="story">
            {headline}
            {byline_h4}
            {self.body_html}
        </article>
        """



import datetime

from uuid import uuid4
from typing import List, Type
from ebooklib import epub

from .styles import Style, AutumnStyle

from .util import PlacementPreference
from .storyprovider import StoryProvider


class Goosepaper:
    def __init__(
        self,
        story_providers: List[StoryProvider],
        title: str = None,
        subtitle: str = None,
    ):
        self.story_providers = story_providers
        self.title = title if title else "Daily Goosepaper"
        self.subtitle = subtitle + "\n" if subtitle else ""
        self.subtitle += datetime.datetime.today().strftime("%B %d, %Y %H:%M")

    def get_stories(self, deduplicate: bool = False):
        stories = []
        for prov in self.story_providers:
            new_stories = prov.get_stories()
            for a in new_stories:
                if deduplicate:
                    for b in stories:
                        if a.headline == b.headline:
                            break
                    else:
                        stories.append(a)
                else:
                    stories.append(a)
        return stories

    def to_html(self) -> str:
        stories = self.get_stories()

        # Get ears:
        ears = [
            s
            for s in stories
            # TODO: Which to prioritize?
            if s.placement_preference == PlacementPreference.EAR
        ]
        right_ear = ""
        left_ear = ""
        if len(ears) > 0:
            right_ear = ears[0].to_html()
        if len(ears) > 1:
            left_ear = ears[1].to_html()

        main_stories = [
            s.to_html()
            for s in stories
            if s.placement_preference
            not in [PlacementPreference.EAR, PlacementPreference.SIDEBAR]
        ]

        sidebar_stories = [
            s.to_html()
            for s in stories
            if s.placement_preference == PlacementPreference.SIDEBAR
        ]

        return f"""
            <html>
            <head>
                <meta
                    http-equiv="Content-type"
                    content="text/html;
                    charset=utf-8" />
                <meta charset="UTF-8" />
            </head>
            <body>
                <div class="header">
                    <div class="left-ear ear">{left_ear}</div>
                    <div><h1>{self.title}</h1><h4>{self.subtitle}</h4></div>
                    <div class="right-ear ear">{right_ear}</div>
                </div>
                <div class="stories row">
                    <div class="main-stories column">
                        {"<hr />".join(main_stories)}
                    </div>
                    <div class="sidebar column">
                        {"<br />".join(sidebar_stories)}
                    </div>
                </div>
            </body>
            </html>
        """

    def to_pdf(
        self,
        filename: str,
        style: Type[Style] = AutumnStyle,
        font_size: int = 14,
    ) -> str:
        """
        Renders the current Goosepaper to a PDF file on disk.

        TODO: If an IO type is provided, write bytes instead.

        """
        from weasyprint import HTML, CSS

        style_obj = style()
        html = self.to_html()
        h = HTML(string=html)
        c = CSS(string=style_obj.get_css(font_size))
        h.write_pdf(filename, stylesheets=[c, *style_obj.get_stylesheets()])
        return filename

    def to_epub(
        self,
        filename: str,
        style: Type[Style] = AutumnStyle,
        font_size: int = 14,
    ) -> str:

        """
        Render the current Goosepaper to an epub file on disk
        """
        stories = []

        for prov in self.story_providers:
            new_stories = prov.get_stories()
            for a in new_stories:
                if not a.headline:
                    stories.append(a)
                    continue
                for b in stories:
                    if a.headline == b.headline:
                        break
                else:
                    stories.append(a)

        book = epub.EpubBook()
        title = f"{self.title} - {self.subtitle}"
        book.set_title(title)
        book.set_language("en")

        style_obj = Style()
        css = epub.EpubItem(
            uid="style_default",
            file_name="style/default.css",
            media_type="text/css",
            content=style_obj.get_css(font_size),
        )
        book.add_item(css)

        chapters = []
        links = []
        no_headlines = []
        for story in stories:
            if not story.headline:
                no_headlines.append(story)
        stories = [x for x in stories if x.headline]
        for story in stories:
            file = f"{uuid4().hex}.xhtml"
            title = story.headline
            chapter = epub.EpubHtml(title=title, file_name=file, lang="en")
            links.append(file)
            chapter.content = story.to_html()
            book.add_item(chapter)
            chapters.append(chapter)

        if no_headlines:
            file = f"{uuid4().hex}.xhtml"
            chapter = epub.EpubHtml(
                title="From Reddit",
                file_name=file,
                lang="en",
            )
            links.append(file)
            chapter.content = "<br>".join([s.to_html() for s in no_headlines])
            book.add_item(chapter)
            chapters.append(chapter)

        book.toc = chapters
        book.add_item(epub.EpubNcx())
        book.add_item(epub.EpubNav())
        book.spine = ["nav"] + chapters

        epub.write_epub(filename, book)
        return filename



from .goosepaper import Goosepaper

from .storyprovider import LoremStoryProvider


def test_can_create_goosepaper_with_no_providers():
    g = Goosepaper([])
    assert g.story_providers == []


def test_can_create_goosepaper_with_duplicate_provider():
    g = Goosepaper([LoremStoryProvider(limit=3), LoremStoryProvider(limit=4)])
    assert len(g.get_stories()) == 7


def test_can_deduplicate_by_headline():
    g = Goosepaper([LoremStoryProvider(limit=3), LoremStoryProvider(limit=4)])
    assert len(g.get_stories(deduplicate=True)) == 1


def test_can_create_html():
    g = Goosepaper([LoremStoryProvider()])
    assert "<html>" in g.to_html()
    assert "Lorem ipsum" in g.to_html()



import feedparser
from typing import List

from .util import PlacementPreference
from .storyprovider import StoryProvider
from .story import Story


class RedditHeadlineStoryProvider(StoryProvider):
    def __init__(self, subreddit: str, limit: int = 20):
        self.limit = limit
        subreddit.lstrip("/")
        subreddit = subreddit[2:] if subreddit.startswith("r/") else subreddit
        self.subreddit = subreddit

    def get_stories(self, limit: int = 20) -> List[Story]:
        feed = feedparser.parse(f"https://www.reddit.com/r/{self.subreddit}.rss")
        limit = min(self.limit, len(feed.entries), limit)
        stories = []
        for entry in feed.entries[:limit]:
            try:
                author = entry.author
            except AttributeError:
                author = "A Reddit user"
            stories.append(
                Story(
                    headline="",
                    body_text=str(entry.title),
                    byline=f"{author} in r/{self.subreddit}",
                    date=entry.updated_parsed,  # type: ignore
                    placement_preference=PlacementPreference.SIDEBAR,
                )
            )
        return stories



import sys
import datetime

from goosepaper.goosepaper import Goosepaper
from goosepaper.util import construct_story_providers_from_config_dict
from goosepaper.upload import upload
from goosepaper.multiparser import MultiParser


def main():
    multiparser = MultiParser()
    config = multiparser.config

    nostory = multiparser.argumentOrConfig("nostory")

    filename = multiparser.argumentOrConfig(
        "output",
        default=f"Goosepaper-{datetime.datetime.now().strftime('%Y-%B-%d-%H-%M')}.pdf",
    )

    if not nostory:  # global nostory flag
        story_providers = construct_story_providers_from_config_dict(config)
        font_size = multiparser.argumentOrConfig("font_size", 14)

        title = config["title"] if "title" in config else None
        subtitle = config["subtitle"] if "subtitle" in config else None

        paper = Goosepaper(
            story_providers=story_providers, title=title, subtitle=subtitle
        )

        if filename.endswith(".html"):
            with open(filename, "w") as fh:
                fh.write(paper.to_html())
        elif filename.endswith(".pdf"):
            paper.to_pdf(filename, font_size=font_size)
        elif filename.endswith(".epub"):
            paper.to_epub(filename, font_size=font_size)
        else:
            print(f"Unknown file extension '{filename.split('.')[-1]}'.")
            exit(1)

    if multiparser.argumentOrConfig("upload"):
        if multiparser.argumentOrConfig("noupload"):
            print(
                "Honk! The 'upload' directive was found, but '--noupload' was also specified on the command line. Your goosepaper {0} was generated but I'm not uploading it.".format(
                    filename
                )
            )
        else:
            upload(filepath=filename, multiparser=multiparser)

    return 0


if __name__ == "__main__":
    sys.exit(main())



import argparse
import pathlib

from goosepaper.util import load_config_file


class NewLineFormatter(argparse.HelpFormatter):
    def _split_lines(self, text, width):
        if text.startswith("||"):
            return text[2:].splitlines()
        return argparse.HelpFormatter._split_lines(self, text, width)


class MultiParser:
    def __init__(self):
        """
        Creates a new MultiParser, which abstracts acessing command line
        arguments and config file entries.

        """

        self.parser = argparse.ArgumentParser(
            prog="goosepaper",
            description="Goosepaper generates and delivers a daily newspaper in PDF format.",
            formatter_class=NewLineFormatter,
        )

        self.parser.add_argument(
            "-c",
            "--config",
            required=False,
            default="",
            help="The json file to use to generate this paper.",
        )
        self.parser.add_argument(
            "-o",
            "--output",
            required=False,
            help="The output file path at which to save the paper",
        )
        self.parser.add_argument(
            "-f",
            "--folder",
            required=False,
            help="Folder to which the document will be uploaded in your remarkable.",
        )
        self.parser.add_argument(
            "-u",
            "--upload",
            action="store_true",
            required=False,
            default=None,
            help="Whether to upload the file to your remarkable using rmapy.",
        )
        self.parser.add_argument(
            "--noupload",
            action="store_true",
            required=False,
            default=None,
            help="Overrides any other 'upload: true' arguments from config files or command line. Useful for testing configs or story generation without having to edit config files.",
        )
        self.parser.add_argument(
            "--showconfig",
            action="store_true",
            required=False,
            default=None,
            help="Print out all config files and command line options in order loaded and the final config to help finding conflicting options. Needed since there are now four possible ways to pass options.",
        )
        self.parser.add_argument(
            "-n",
            "--nostory",
            required=False,
            default=False,
            action="store_true",
            help='||Skip story creation. Combined with "--upload" can be used to\nupload a preexisting output file.\n\n** NOTE ** If used without "--upload" goosepaper will run but\nperform no action.',
        )
        self.parser.add_argument(
            "--replace",
            action="store_true",
            required=False,
            default=None,
            help="||Will replace a document with same name in your remarkable.\nDefault behaviour is case sensitive (ala *nix/Mac).\n\ne.g. 'A Flock of RSS Feeds.epub' and 'a flock of rss feeds.epub'\nare seen as TWO different files. Can be altered with '--nocase'\nor '--strictlysane' switches.",
        )
        self.parser.add_argument(
            "--noreplace",
            action="store_true",
            required=False,
            default=None,
            help="Only valid when specified on command line (ignored if present in any config file). Supersedes any config file setting for 'replace: true', thus ensuring that the file will NEVER be overwritten. Will also supersede command line '--replace' if both are specified regardless of order.",
        )
        self.parser.add_argument(
            "--cleanup",
            required=False,
            default=None,
            action="store_true",
            help="Delete the output file after upload.",
        )
        self.args = self.parser.parse_args()

        # These are in order of precedence, low to highest
        #  1. Home directory global configs
        #  2. Local directory from which goosepaper is called
        #  3. Specified on the command line.

        defaultconfigs = [
            str(pathlib.Path("~").expanduser()) + "/.goosepaper.json",
            "./goosepaper.json",
            self.args.config,
        ]
        self.config = {}
        outputcount = 0
        debug_configs = True if self.args.showconfig else None

        # Debug code for troubleshooting config file and cli override issues.
        if debug_configs:
            import pprint

            pp = pprint.PrettyPrinter(indent=3)
            print(
                "\n".join(
                    [
                        "Command line arguments received:",
                        "(including default values)",
                        "--------------------------------",
                    ]
                )
            )
            pp.pprint(self.args)

        # If passed a config file on the command line, assume it's important
        # so fail if not readable.

        if self.args.config:
            try:
                load_config_file(self.args.config)
            except FileNotFoundError:
                print(
                    f"Couldn't find config file ({self.args.config}) "
                    "specified on the command line. Aborting."
                )
                exit(1)

        for defconfigfile in defaultconfigs:
            try:
                tempconfig = load_config_file(defconfigfile)
                if "output" in tempconfig and "output" in self.config:
                    outputcount = outputcount + 1
                if "stories" in tempconfig and "stories" in self.config:
                    for story in self.config["stories"]:
                        tempconfig["stories"].append(story)

                self.config.update(tempconfig)
                if debug_configs:
                    print(
                        f"\nConfig options found in {defconfigfile}:"
                        "\n---------------------\n"
                    )
                    pp.pprint(load_config_file(defconfigfile))
            except FileNotFoundError:
                pass

        if debug_configs:
            print("\nFinal config values are:\n------------------")
            pp.pprint(self.config)
            print("")

        if "noreplace" in self.config:
            del self.config["noreplace"]

    def argumentOrConfig(self, key, default=None, dependency=None):
        """
        Returns a command line argument or an entry from the config file

        Arguments:
            key: the command line option name (as in --key) or config entry
            default (str: None): the default value, returned if the key was not
                set both as a command line argument and a config entry
            dependency (str: None): the name of a dependency command line
                argument or config entry that must be present for this call to
                be valid

        Returns:
            If a command line option with 'key' name was set, returns it. Else,
                if a config entry named 'key' was set, returns it. If none of
                the previous was returned, returns the default value specified
                by the 'default' argument.

        """

        d = vars(self.args)
        if key in d and d[key] is not None:
            if dependency and dependency not in d:
                self.parser.error(f"--{key} requires --{dependency}.")
            value = d[key]
        elif key in self.config:
            value = self.config[key] or default
        else:
            value = default

        return value
